package Catalyst::Action::CrossSiteREST;

use strict;
use warnings;

use base 'Catalyst::Action::REST';

BEGIN { require 5.008001; }

our $VERSION = '0.1';
$VERSION = eval $VERSION;

# To allow cross-site requests, we need to return the relevant headers on
# the OPTIONS response in order to persuade the user agent to issue the
# request. However, we don't want to write this support for every request.
# So we override the generic handling.
sub _return_options {
  my ($self, $method_name, $controller, $c) = @_;
  my @allowed = $self->_get_allowed_methods($controller, $c, $method_name);
  $c->response->content_type('text/plain');
  $c->response->status(200);
  $c->response->header('Allow' => \@allowed);
  
  # Access Control
  $c->response->header("Access-Control-Allow-Origin", "*");
  $c->response->header('Access-Control-Allow-Methods' => \@allowed);
  # Work around JS library brokenness - JQuery sends a custom header on all 
  # requests, as did Dojo until very recently. We allow whatever headers they
  # want to send.
  $c->response->header('Access-Control-Allow-Headers' => 
                       $c->request->header("Access-Control-Request-Headers"));
}

1;
