use strict;
use warnings;
use Test::More;

# ctx_request is new in Catalyst 5.8, but deployment server runs 5.7
if (!defined(&main::ctx_request)) {
  plan skip_all => 'No ctx_request function in this version of Catalyst';
}

BEGIN { use Catalyst::Test 'Bugzilla::API' }

plan(tests => 9);

use_ok 'Bugzilla::API::Model::Utils';

# Get a context object for passing to functions; doesn't matter what the
# request is.
my($res, $c) = ctx_request('/bug/1');

##############################################################################
# (un)?fix_time() functions
##############################################################################
my $bug;
$bug = { 'time' => "2009-06-06 03:10:10 -0700" };
fix_time($c, $bug, 'time');
is($bug->{'time'}, "2009-06-06T10:10:10Z", "Fixed 1");

$bug = { 'time' => "This is not a valid time" };
fix_time($c, $bug, 'time');
like($bug->{'time'}, qr/str2time failed/, "Error given when parse fails");

$c->config->{'bugzilla_timezone'} = "America/Los_Angeles";

$bug = { 'time' => "2009-06-06T10:10:10Z" };
unfix_time($c, $bug, 'time');
is($bug->{'time'}, "2009-06-06 03:10:10", "Unfixed 1");

##############################################################################
# fix_person() function
##############################################################################

my $correct = { 
  'person' => { 
    'name' => 'test',
  } 
};

my $correct_li = { 
  'person' => { 
    'name' => 'test@example.com',
    'ref' => 'http://localhost/user/test@example.com' 
  } 
};

my $correct_rn = { 
  'person' => {
    'name'      => 'test', 
    'real_name' => 'Test'
  }
};

my $correct_rn_li = { 
  'person' => {
    'name'      => 'test@example.com', 
    'ref' => 'http://localhost/user/test@example.com',
    'real_name' => 'Test'
  }
};

my $input;

$input = { 'person' => 'test' };
fix_person($c, $input, "person");
is_deeply($input, $correct, "Straight string (not logged in)");

$input = { 'person' => 'test@example.com' };
fix_person($c, $input, "person");
is_deeply($input, $correct_li, "Straight string (logged in)");

$input = { 'person' => 'test', 'person_realname' => 'Test' };
fix_person($c, $input, "person");
is_deeply($input, $correct_rn, "Real name (not logged in)");

$input = { 'person' => 'test@example.com', 'person_realname' => 'Test' };
fix_person($c, $input, "person");
is_deeply($input, $correct_rn_li, "Real name (logged in)");

# Should be no change
$input = { 
  'person' => {
    'name'      => 'test@example.com', 
    'ref' => 'http://localhost/user/test@example.com',
    'real_name' => 'Test'
  }
};

fix_person($c, $input, "person");
is_deeply($input, $correct_rn_li, "No change on correct person");

done_testing();
