use strict;
use warnings;

use FindBin '$Bin';
use lib "$Bin/lib";    
use Test::More;

use JSON;
use Slurp;
use Test::Rest;
use TestUtils;
use Data::Dumper;
use HTTP::Request::Common;
use LWP::UserAgent;

BEGIN { use Catalyst::Test 'Bugzilla::API' }

my ($res, $req, $old, $new, $json);
my $t = Test::Rest->new('content_type' => 'application/json');
##############################################################################

plan(tests => 5);

##############################################################################
# Check Bugzilla is up
##############################################################################
my $config = slurp("$Bin/../bugzilla_api.conf");
$config =~ /\n\s*bugzilla_url\s+([^\s]+)/s;
my $bugzilla_url = $1;

my $ua = LWP::UserAgent->new;

my $url = $bugzilla_url . "query.cgi";

$res = $ua->get($url);
ok($res->is_success, "Bugzilla up OK");

##############################################################################
# Check BzAPI is compiled, running and doing simple requests
##############################################################################
use_ok 'Bugzilla::API::Model::Bug';

$res = request('/bug/1');
ok($res->is_success, "Request OK for /bug/1");

like($res->decoded_content, 
     qr/qa_contact/, 
     "QA contact present (is it enabled? did you copy data/params over?)");

ok(request('/count')->is_success, 
   'BzAPI extension installed (check by asking for /count)');
