use strict;
use warnings;

use FindBin '$Bin';
use lib "$Bin/lib";    
use Test::More;

use JSON;
use Slurp;
use Test::Rest;
use TestUtils;
use Data::Dumper;
use HTTP::Request::Common;
use LWP::UserAgent;

BEGIN { use Catalyst::Test 'Bugzilla::API' }

my ($res, $req, $old, $new, $json);
my $t = Test::Rest->new('content_type' => 'application/json');
##############################################################################

my $get_and_check_ct = 2;

plan(tests => (($get_and_check_ct * 2) + 2));

sub get_and_check {
  my ($path, $filename, $cookie) = @_;
  
  my $url = $path . "?userid=1&cookie=" . $cookie;
  $old = load("$filename");

  $res = request($url);
  ok($res->is_success, "Request OK for $url");
  $new = from_json($res->decoded_content, {utf8 => 1});

  is_deeply(canonicalize($new), canonicalize($old), "Data OK for $url");
}

##############################################################################
# Do login via HTTP interface to get cookie
##############################################################################
my $config = slurp("$Bin/../bugzilla_api.conf");
$config =~ /\n\s*bugzilla_url\s+([^\s]+)/s;
my $bugzilla_url = $1;

my $ua = LWP::UserAgent->new;

# Username needs URL escaping. Quick and dirty.
my $un = $USERNAME;
$un =~ s/@/%40/;

my $url = $bugzilla_url . "index.cgi?" . 
                          "Bugzilla_login=$un&" .  
                          "Bugzilla_password=$PASSWORD&GoAheadAndLogIn=Log+in";

$res = $ua->get($url);
ok($res->is_success, "Login request OK");

# Snarf cookie
# Header looks like this:
# Set-Cookie: Bugzilla_logincookie=kxVcJxLZKV; path=/; expires=Fri, 01-Jan-2038 00:00:00 GMT; HttpOnly

my @cookie_headers = $res->header("Set-Cookie");
my $cookie;

foreach my $header (@cookie_headers) {
  if ($header =~ /Bugzilla_logincookie=([^;]+);/) {
    $cookie = $1;
  }
}

isnt($cookie, undef, "Cookie found");

##############################################################################
# Do GET-backed request and check for logged-in data
##############################################################################
get_and_check("/bug/1", "bug1-li.json", $cookie);

##############################################################################
# Do XML-RPC-backed request and check for logged-in data
##############################################################################
get_and_check("/bug/12/history", "history-li.json", $cookie);
