use strict;
use warnings;

use FindBin '$Bin';
use lib "$Bin/lib";    
use Test::More;

use JSON;
use Slurp;
use Test::Rest;
use TestUtils;
use Data::Dumper;

BEGIN { use Catalyst::Test 'Bugzilla::API' }

my ($res, $req, $old, $new, $json);
my $t = Test::Rest->new('content_type' => 'application/json');
##############################################################################

my %allow = (
  "/bug"                    => "GET, POST",
  "/bug/1"                  => "GET, PUT",
  "/bug/1/flag"             => "GET",
  "/bug/1/comment"          => "GET, POST",
  "/bug/1/history"          => "GET",
  "/bug/1/attachment"       => "GET, POST",
  "/attachment/1"           => "GET, PUT",
  "/user"                   => "GET",
  "/user/gerv\@mozilla.org" => "GET",
  "/configuration"          => "GET",
);

plan(tests => (3 * scalar(keys %allow)) + (2 * (scalar(keys %allow) - 2)));

foreach my $url (keys %allow) {
  my $allow = $allow{$url};

  # OPTIONS
  $req = $t->options(url => $url);

  $res = request($req);
  ok($res->is_success, "OPTIONS request OK for $url");
  is($res->header("Allow"), $allow, "Allow header correct for $url");
  is($res->header("Access-Control-Allow-Origin"), "*", 
     "ACAO header correct for $url");
     
  # GET
  next if $url eq "/bug"; # Avoid downloading all bugs in the installation!
  next if $url eq "/user"; # Listing all users requires a login
  
  $res = request($url);
  ok($res->is_success, "Request OK for $url");
  is($res->header("Access-Control-Allow-Origin"), "*", 
     "ACAO header correct for $url");
}

done_testing();
