use strict;
use warnings;

use FindBin '$Bin';
use lib "$Bin/lib";    
use Test::More;

use JSON;
use Slurp;
use Test::Rest;
use TestUtils;
use Data::Dumper;

BEGIN { use Catalyst::Test 'Bugzilla::API' }

my ($res, $req, $old, $new, $json);
my $t = Test::Rest->new('content_type' => 'application/json');
##############################################################################

my $get_and_check_ct = 3;

plan(tests => ($get_and_check_ct * 4) + 1);

sub get_and_check {
  my ($url, $filename, $count) = @_;
  
  my $old = load("$filename");

  $res = request($url);
  ok($res->is_success, "Request OK for $url");
  $new = from_json($res->decoded_content, {utf8 => 1});
  
  # Demote if we have an array back
  if ($new->{'bugs'}) {
    my @bugs = @{ $new->{'bugs'} };
    
    is(scalar(@bugs), $count, "Right number of bugs returned");
    
    # Put bugs in a predictable order
    @bugs = sort { $a->{'id'} <=> $b->{'id'} } @bugs;
    
    $new = $bugs[0];
  }
  else {
    # Even out test count
    is (0, 0, "");
  }

  is_deeply(canonicalize($new), canonicalize($old), "Bug OK for $url");
}

##############################################################################
# One bug
# We test both the direct getting of a bug and the special getting via a
# search URL specifying a bug ID only
##############################################################################
get_and_check('/bug/1', "bug1.json");

##############################################################################
# One bug, logged in
##############################################################################
get_and_check('/bug/1?' . $LOGIN, "bug1-li.json");

##############################################################################
# Bug + all optional bits
##############################################################################
get_and_check('/bug/1?include_fields=_default,comments,history&attachmentdata=1&' . $LOGIN, "bug1-all.json");
get_and_check('/bug/1?include_fields=_default,comments,history,attachments.data,attachments.encoding&' . $LOGIN, "bug1-all.json");

##############################################################################
# Bad bug ID
##############################################################################
$res = request('/bug/a');
$json = from_json($res->decoded_content, {utf8 => 1});

is($json->{'error'}, JSON::true, "Get error");
