use strict;
use warnings;

use FindBin '$Bin';
use lib "$Bin/lib";    
use Test::More;

use JSON;
use Slurp;
use Test::Rest;
use TestUtils;
use Data::Dumper;

BEGIN { use Catalyst::Test 'Bugzilla::API' }

my ($res, $req, $old, $new, $json);
my $t = Test::Rest->new('content_type' => 'application/json');
##############################################################################

plan(tests => 14);

##############################################################################
# List all bugs
##############################################################################
$res = request('/bug');

ok($res->is_success, 'Request should succeed');
$json = from_json($res->decoded_content, {utf8 => 1});
ok($json->{'bugs'}, "Got bug object");
ok(scalar(@{$json->{'bugs'}}) > 5, "Got some bugs");

##############################################################################
# List just some bugs
##############################################################################
$res = request('/bug?id=1,2,3,4,5,6&order=Bug%20Number');

ok($res->is_success, 'Request should succeed');
$json = from_json($res->decoded_content, {utf8 => 1});
ok($json->{'bugs'}, "Got bug array");

is(scalar(@{$json->{'bugs'}}), 6, "Right number of bugs");
is($json->{'bugs'}->[5]->{'id'}, "6", "Last bug is 6");

my $bug1 = load("bug-bug1.json");

is_deeply(canonicalize($json->{'bugs'}->[0]), 
          canonicalize($bug1->{'bugs'}->[0]), "Bug 1 is correct");

##############################################################################
# Do a search which returns no bugs
##############################################################################
$res = request('/bug?id=100000');

ok($res->is_success, 'Request should succeed');
$json = from_json($res->decoded_content, {utf8 => 1});
ok($json->{'bugs'}, "Got bug array");
is(scalar(@{$json->{'bugs'}}), 0, "No bugs from search");

##############################################################################
# List some bugs when logged in
##############################################################################
$res = request('/bug?id=1,2,3,4,5,6&order=Bug%20Number&' . $LOGIN);
$json = from_json($res->decoded_content, {utf8 => 1});
ok($json->{'bugs'}, "Got bug array");
is(scalar(@{$json->{'bugs'}}), 6, "Right number of bugs");

$bug1 = load("bug-bug1-li.json");

is_deeply(canonicalize($json->{'bugs'}->[0]), 
          canonicalize($bug1->{'bugs'}->[0]), "Bug 1 (logged in) is correct");

done_testing();
