use strict;
use warnings;

use FindBin '$Bin';
use lib "$Bin/lib";    
use Test::More;

use JSON;
use Slurp;
use Test::Rest;
use TestUtils;
use Data::Dumper;

BEGIN { use Catalyst::Test 'Bugzilla::API' }

my ($res, $req, $old, $new, $json);
my $t = Test::Rest->new('content_type' => 'application/json');
##############################################################################

##############################################################################
# 100 Ways To Get A Search Error - or not? Bugzilla seems to check search 
# types (with the exception of boolean charts) but not values. It also checks
# the field name of chfield but not those of boolean charts.
##############################################################################
my @urls_nobugs = (
  '/bug?id=Phooey',
  '/bug?id=1&classification=Phooey',
  '/bug?id=1&component=Phooey',
  '/bug?id=1&op_sys=Phooey',
  '/bug?id=1&platform=Phooey',
  '/bug?id=1&priority=Phooey',
  '/bug?id=1&product=Phooey', 
  '/bug?id=1&resolution=Phooey',
  '/bug?id=1&severity=Phooey',
  # XXX Changed behaviour in 4.1+; under discussion (1 error)
  # https://bugzilla.mozilla.org/show_bug.cgi?id=601093
  # '/bug?id=1&status=Phooey', 
  '/bug?id=1&target_milestone=Phooey',
  '/bug?id=1&version=Phooey',
  '/bug?id=1&email1=Phooey@gerv.net&email1_creator=1&email1_type=equals',
);

my @urls_400 = (
  '/bug?id=1&keywords=Phooey,tearjerking&keywords_type=contains_all_words',
  '/bug?id=1&comment=bowling&comment_type=Phooey', 
  '/bug?id=1&keywords=tearjerking&keywords_type=Phooey',
  '/bug?id=1&summary=Christmas%20Phooey&summary_type=Phooey',
  '/bug?id=1&url=http%3A%2F%2Fwww.Phooeypin.co.uk%2F&url_type=Phooey',
  '/bug?id=1&whiteboard=%5BPhooey%3AP1%5D&whiteboard_type=Phooey',
  '/bug?id=1&email1=gerv@gerv.net&email1_creator=1&email1_type=Phooey',
  '/bug?id=1&email2=Phooey@gerv.net&email2_creator=1&email2_type=Phooey',
  # changed_before/changed_after are not validated by Bugzilla
  # deadline_before/deadline_after are not validated by Bugzilla
  '/bug?id=1&field0-0-0=Phooey&type0-0-0=equals&value0-0-0=something',
  '/bug?id=1&field0-0-0=alias&type0-0-0=Phooey&value0-0-0=something',
);

plan(tests => (@urls_nobugs * 2) + (@urls_400 * 3));

foreach my $url (@urls_nobugs) {
  $res = request($url);
  ok($res->is_success, "Request OK for $url");
  $new = from_json($res->decoded_content, {utf8 => 1});
  is_deeply($new->{'bugs'}, [], "Empty bugs array returned by $url");
}

foreach my $url (@urls_400) {
  $res = request($url);
  is($res->code, '400', "HTTP code 400 for $url");
  $new = from_json($res->decoded_content, {utf8 => 1});
  is($new->{'error'}, JSON::true, "Error object returned by $url");
  ok(exists($new->{'message'}), "Message present in error returned by $url");
}

done_testing();
