use strict;
use warnings;

use FindBin '$Bin';
use lib "$Bin/lib";    
use Test::More;

use JSON;
use Slurp;
use Test::Rest;
use TestUtils;
use Data::Dumper;

BEGIN { use Catalyst::Test 'Bugzilla::API' }

my ($res, $req, $old, $new, $json);
my $t = Test::Rest->new('content_type' => 'application/json');
##############################################################################

my @urls;

##############################################################################
# 100 Ways To Find/Not Find Bug #1
##############################################################################
my $bug1 = load("bug-bug1.json");
$bug1 = canonicalize($bug1->{'bugs'}->[0]);
my $bug1_li = load("bug-bug1-li.json");
$bug1_li = canonicalize($bug1_li->{'bugs'}->[0]);

@urls = (
  '/bug?id=1&classification=***', 'Places', 'Software',
  '/bug?id=1&component=***', 'Bedrock%20Bowling', 'Paper%20Delivery',
  '/bug?id=1&op_sys=***', 'Linux', 'All',
  '/bug?id=1&platform=***', 'PC', 'All',
  '/bug?id=1&priority=***', 'P5', 'P1',
  '/bug?id=1&product=***', 'Bedrock', 'Hollyrock',
  '/bug?id=1&resolution=***', '---', 'FIXED',
  '/bug?id=1&severity=***', 'enhancement', 'critical',
  '/bug?id=1&status=***', 'NEW', 'RESOLVED',
  '/bug?id=1&target_milestone=***', 'Many%20Beta', 'Many%20Alpha',
  '/bug?id=1&version=***', 'Two', 'One',
  '/bug?id=1&comment=bowling%20***&comment_type=contains_all_words', 'alley', 'hall',
  '/bug?id=1&comment=bowling%20***', 'alley', 'hall',
# XXX Broken: https://bugzilla.mozilla.org/show_bug.cgi?id=490322
#  '/bug?id=1&keywords=***,tearjerking&keywords_type=contains_all_words', 'charming', 'interesting',
#  '/bug?id=1&keywords=***,tearjerking', 'charming', 'interesting',
  '/bug?id=1&summary=Christmas%20***&summary_type=contains_all_words', 'Tree', 'Paper',
  '/bug?id=1&summary=Christmas%20***', 'Tree', 'Paper',
  '/bug?id=1&url=http%3A%2F%2Fwww.***pin.co.uk%2F&url_type=equals', 'ten', 'nine',
  '/bug?id=1&url=http%3A%2F%2Fwww.***pin.co.uk%2F', 'ten', 'nine',
  '/bug?id=1&whiteboard=%5B***%3AP1%5D&whiteboard_type=equals', 'freddie', 'Wilma',
  '/bug?id=1&whiteboard=%5B***%3AP1%5D', 'freddie', 'Wilma',
  '/bug?id=1&email1=***@gerv.net&email1_creator=1&email1_type=equals', 'fred', 'barney',
  '/bug?id=1&email1=***@gerv.net&email1_assigned_to=1&email1_type=equals', 'fred', 'barney',
  '/bug?id=1&email1=***@gerv.net&email1_qa_contact=1&email1_type=equals', 'barney', 'fred',
  '/bug?id=1&email1=***@gerv.net&email1_comment_creator=1&email1_type=equals', 'fred', 'mrslate',
  '/bug?id=1&email1=***@gerv.net&email1_cc=1&email1_type=equals', 'barney', 'mrslate',
  '/bug?id=1&email2=***@gerv.net&email2_creator=1&email2_type=equals', 'fred', 'barney',
  '/bug?id=1&changed_after=2009-***-28&changed_before=2009-***-30', '09', '01',
  '/bug?id=1&changed_after=2009-09-22&changed_before=2009-09-24&changed_field=whiteboard&changed_field_to=[***%3AP1]', 'Freddie', 'Wilma',
  '/bug?id=1&changed_after=***-09-21&changed_before=***-09-24&changed_field=creation_time', '2009', '2001',
  '/bug?id=1&deadline_after=2009-***-02&deadline_before=2009-***-04&' . $LOGIN, '10', '11',
  '/bug?id=1&field0-0-0=alias&type0-0-0=equals&value0-0-0=***tree', 'christmas', 'fir',
  # name=value translation
  '/bug?id=1&alias=***tree', 'christmas', 'fir',
  '/bug?id=1&field0-0-0=comment.creator&type0-0-0=equals&value0-0-0=***@gerv.net', 'fred', 'mrslate',
  '/bug?id=1&comment.creator=***@gerv.net', 'fred', 'mrslate',
  '/bug?id=1&field0-0-0=comment.is_private&type0-0-0=equals&value0-0-0=***&' . $LOGIN, '1', '',
  # '/bug?id=1&field0-0-0=content&type0-0-0=matches&value0-0-0=Christmas%20***', 'Tree', 'Box',
  # XXX no full text indexing???
  '/bug?id=1&field0-0-0=flag&type0-0-0=equals&value0-0-0=purple***', '?', '+',
  '/bug?id=1&field0-0-0=flag.setter&type0-0-0=equals&value0-0-0=***', 'gerv@mozilla.org', 'mrslate@gerv.net',
  '/bug?id=1&flag.setter=***', 'gerv@mozilla.org', 'mrslate@gerv.net',
  '/bug?id=1&field0-0-0=idle&type0-0-0=greater_than&value0-0-0=***', '3', '30000',
  '/bug?id=1&field0-0-0=assignee_idle&type0-0-0=greater_than&value0-0-0=***', '4d', '40000d',,
  '/bug?id=1&field0-0-0=attachment.attacher&type0-0-0=equals&value0-0-0=***@gerv.net', 'fred', 'babypuss',
  # Can't easily search attachment data; both bug 1 attachments are binary. Oops.
  # '/bug?id=1&field0-0-0=attachment.data&type0-0-0=contains&value0-0-0=***', '', '',
  '/bug?id=1&field0-0-0=attachment.description&type0-0-0=contains_all&value0-0-0=***', 'Correct', 'Incorrect',
  '/bug?id=1&field0-0-0=attachment.file_name&type0-0-0=equals&value0-0-0=***Pins.jpg', 'ten', 'nine',
  '/bug?id=1&field0-0-0=flag&type0-0-0=equals&value0-0-0=review***', '?', '-',
  '/bug?id=1&field0-0-0=attachment.is_obsolete&type0-0-0=equals&value0-0-0=***', '0', '1',
  '/bug?id=1&field0-0-0=attachment.is_patch&type0-0-0=equals&value0-0-0=***', '0', '1',
  '/bug?id=1&field0-0-0=attachment.is_private&type0-0-0=equals&value0-0-0=***', '0', '1',
  '/bug?id=1&field0-0-0=attachment.content_type&type0-0-0=equals&value0-0-0=image/***', 'gif', 'none',
  # Testing auto-conversion of unrecognised fields to boolean charts
  '/bug?id=1&comment.creator=***@gerv.net', 'fred', 'mrslate',
  '/bug?id=1&cf_textfield=***', 'Aargh!%20%3Ctears%20out%20hair%3E', 'Wibble',
  '/bug?id=1&alias=***tree', 'christmas', 'fir',
);

# Extract out the number of tests we are actually going to run - 
# one set of 3 for every 2nd/3rd entry in the triple which isn't ''.
my @members = grep { $_ ne '' && $_ !~ /bug\?/ } @urls;

plan(tests => (@members * 3));

while (@urls) {
  my ($pattern, $right, $wrong) = (shift(@urls), shift(@urls), shift(@urls));
  
  # Right
  if ($right ne '') {
    my $url = $pattern;
    $url =~ s/\*\*\*/$right/g;
    $res = request($url);
    ok($res->is_success, "Request OK for $url");
    
    if ($res->is_success) {
      $new = from_json($res->decoded_content, {utf8 => 1});
      is(scalar(@{$new->{'bugs'}}), 1, "Just one bug for $url");
      $old = ($url =~ /username=/ ? $bug1_li : $bug1);
      is_deeply(canonicalize($new->{'bugs'}->[0]), $old, "Bug 1 OK for $url");
    }
  }
  
  # Wrong
  if ($wrong ne '') {
    my $url = $pattern;
    $url =~ s/\*\*\*/$wrong/g;  
    $res = request($url);
    ok($res->is_success, "Request OK for $url");
    
    if ($res->is_success) {
      $new = from_json($res->decoded_content, {utf8 => 1});
      
      # Two tests so as to have same number of tests for both halves
      is(scalar(@{$new->{'bugs'}}), 0, "No bugs for url $url");  
      is(1, 1);  
    }
  }
}

done_testing();
