use strict;
use warnings;

use FindBin '$Bin';
use lib "$Bin/lib";    
use Test::More;

use JSON;
use Slurp;
use Test::Rest;
use TestUtils;
use Data::Dumper;

BEGIN { use Catalyst::Test 'Bugzilla::API' }

my ($res, $req, $old, $new, $json);
my $t = Test::Rest->new('content_type' => 'application/json');
##############################################################################

my $bugid = "53";

my $put_comment_and_check_ct = 2 + $put_and_get_ct;

plan(tests => $put_and_get_ct + 4 + $put_and_get_ct + 2 + (2 * $put_comment_and_check_ct));

my $url = "/bug/$bugid?include_fields=_default,comments&" . $LOGIN;

##############################################################################
# Get bug and count comments
##############################################################################
$res = request($url);  
$old = from_json($res->decoded_content, {utf8 => 1});
my $comment_count = @{$old->{'comments'}};

##############################################################################
# Public comment, privacy flag twiddles
##############################################################################
$old->{'comments'}->[0]->{'is_private'} = JSON::true;
$old->{'comments'}->[1]->{'is_private'} = JSON::false;
my $newcomment = { "text" => "Public comment", "is_private" => JSON::false };
push(@{ $old->{'comments'} }, $newcomment);

my $oldjson = to_json($old, {utf8 => 1});
my $newjson = put_and_get($url, $oldjson, "Update bug with new public comment");
$comment_count++;
$new = from_json($newjson, {utf8 => 1});

is(scalar(@{$new->{'comments'}}), $comment_count, "Comment count OK post-pub");
is($new->{'comments'}->[-1]->{'is_private'}, JSON::false, "Privacy -1 set");
is($new->{'comments'}->[0]->{'is_private'}, JSON::true, "Privacy 0 set");
is($new->{'comments'}->[1]->{'is_private'}, JSON::false, "Privacy 1 set");

##############################################################################
# Private comment, privacy flag twiddles
##############################################################################
$old = $new;

$old->{'comments'}->[0]->{'is_private'} = JSON::false;
$old->{'comments'}->[1]->{'is_private'} = JSON::true;
$newcomment = { "text" => "Private comment", "is_private" => JSON::true };
push(@{ $old->{'comments'} }, $newcomment);

$oldjson = to_json($old, {utf8 => 1});
$newjson = put_and_get($url, $oldjson, "Update bug with new private comment");
$comment_count++;
$new = from_json($newjson, {utf8 => 1});
is(scalar(@{$new->{'comments'}}), $comment_count, "Comment count OK post-priv");
is($new->{'comments'}->[-1]->{'is_private'}, JSON::true, 
                                                 "Privacy of last comment set");

##############################################################################
# Post a single comment without the rest
##############################################################################
sub put_comment_and_check {
  my ($newcomment, $name) = @_;
  
  $oldjson = to_json($newcomment, {utf8 => 1});
  $newjson = put_and_get($url, $oldjson, "Update bug: $name");
  $comment_count++;
  $new = from_json($newjson, {utf8 => 1});
  is(scalar(@{$new->{'comments'}}), $comment_count, "Comment count OK: $name");
  is($new->{'comments'}->[-1]->{'is_private'}, JSON::false, "Privacy -1 set");
}

$newcomment = 
          { "comments" => [ { "text" => "Solo comment", 
                              "is_private" => JSON::false } ] };
put_comment_and_check($newcomment, "One comment");

##############################################################################
# Post two comments and check we only get one
##############################################################################
$newcomment = 
          { "comments" => [ { "text" => "1 of 2 comments", 
                              "is_private" => JSON::false },
                            { "text" => "2 of 2 comments", 
                              "is_private" => JSON::false }
                          ] 
          };
put_comment_and_check($newcomment, "Two comments");

done_testing();
