use strict;
use warnings;

use FindBin '$Bin';
use lib "$Bin/lib";    
use Test::More;

use JSON;
use Slurp;
use Test::Rest;
use TestUtils;
use Data::Dumper;

BEGIN { use Catalyst::Test 'Bugzilla::API' }

my ($res, $req, $old, $new, $json);
my $t = Test::Rest->new('content_type' => 'application/json');
##############################################################################

my $create_bug_ct = 3;
plan(tests => (2 * ($create_bug_ct + 2) + $create_bug_ct + 4));

sub create_bug {
  my ($json) = @_;
  
  $req = $t->post(url => '/bug?' . $LOGIN, 
                  data => $json);

  $res = request($req);
  is($res->code, 201, "Created code returned");

  # Find bug number
  my $created = from_json($res->decoded_content, {utf8 => 1});

  $created->{'ref'} =~ /(\d+)$/;
  like($1, qr/^\d+$/, "Found bug number");
  $new = {};

  if ($1) {
    $res = request("/bug/$1?include_fields=_default,comments&" . $LOGIN);

    ok($res->is_success, 'Request should succeed');
    $new = from_json($res->decoded_content, {utf8 => 1});
  }
  
  return $new;
}

##############################################################################
# Creation test - full
##############################################################################
foreach my $letter ("A", "C") {
  $json = slurp("$Bin/json/create-$letter.json");
  $new = create_bug($json);
  $old = from_json($json, {utf8 => 1});

  is($new->{'comments'}->[0]->{'text'}, 
     $old->{'comments'}->[0]->{'text'}, "Comments match ($letter)");

  # XXX custom field support (cf_single)
  my $bugid = $new->{'id'};
  my @dontcompare = ('last_change_time', 'creation_time', 'creator', 'id', 
                     'ref', 'update_token', 'description', 'comments', 
                     'actual_time', 'attachment', 'cf_single');
  foreach my $field (@dontcompare) {
    delete($old->{$field});
    delete($new->{$field});
  }

  is_deeply($new, $old, "Created bug " . $bugid . " (from $letter) is correct");
}

##############################################################################
# Creation test - minimal
##############################################################################
$json = slurp("$Bin/json/create-B.json");
$new = create_bug($json);
$old = from_json($json, {utf8 => 1});

foreach my $field ("product", "component", "version", "summary") {
  is($new->{$field}, $old->{$field}, "$field OK");
}
