use strict;
use warnings;

use FindBin '$Bin';
use lib "$Bin/lib";    
use Test::More;

use JSON;
use Slurp;
use Test::Rest;
use TestUtils;
use Data::Dumper;

BEGIN { use Catalyst::Test 'Bugzilla::API' }

my ($res, $req, $old, $new, $json);
my $t = Test::Rest->new('content_type' => 'application/json');
##############################################################################

my $bugid = "21";

my $reset_flags_ct = 2;

plan(tests => $reset_flags_ct + ($put_and_check_flags_ct * 3) + $put_and_get_ct + 1 + $reset_flags_ct);

my $url = "/bug/$bugid?" . $LOGIN;

sub reset_flags {
  my $otherway = slurp("$Bin/json/flags-otherway.json");
  $old = from_json($otherway, {utf8 => 1});
  
  $res = request($url);
  my $bug = from_json($res->decoded_content, {utf8 => 1});
  foreach my $flag (@{$bug->{'flags'}}) {
    if ($flag->{'id'} == 152) {
      $flag->{'status'} = "+";
    }
    elsif ($flag->{'id'} == 153) {
      $flag->{'status'} = "?";
      $flag->{'requestee'}->{'name'} = 'fred@gerv.net';
    }
    else {
      $flag->{'status'} = "X";
    }
  }

  $req = $t->put(url  => $url, 
                 data => to_json($bug, {utf8 => 1}));

  $res = request($req);
  my $ok = from_json($res->decoded_content, {utf8 => 1});
  is($ok->{'ok'}, 1, "PUT OK");
  
  # Check it's worked
  $res = request($url);  
  $new = from_json($res->decoded_content, {utf8 => 1});
  $new = { 'flags' => $new->{'flags'} };
  
  is_deeply(canonicalize($new), canonicalize($old), 
                                      "Bug flags restored to starting state");
}

reset_flags();

##############################################################################
# Twiddle existing flags
##############################################################################
my $oneway = slurp("$Bin/json/flags-oneway.json");
my $otherway = slurp("$Bin/json/flags-otherway.json");

put_and_check_flags($url, $oneway, "One way (single)");
put_and_check_flags($url, $otherway, "Other way (single)");

##############################################################################
# Add new flags
##############################################################################
my $additional = slurp("$Bin/json/flags-additional.json");
put_and_check_flags($url, $additional, "Adding flags", 1);

##############################################################################
# Don't remove flags - flag not present in update should be left alone
##############################################################################
my $newjson = put_and_get($url, $otherway, "Not removing added flags");
$old = from_json($additional, {utf8 => 1});
$new = from_json($newjson, {utf8 => 1});
$new = { 'flags' => $new->{'flags'} };

foreach my $flag (@{$new->{'flags'}}) {
  next if $flag->{'id'} =~ /^152|153$/;
  delete($flag->{'id'});
  delete($flag->{'setter'});
}

is_deeply(canonicalize($new), canonicalize($old), 
                                              "Flags correct for dont-remove");

##############################################################################
# Remove the new flags by setting to X
##############################################################################
reset_flags();

done_testing();
