use strict;
use warnings;

use FindBin '$Bin';
use lib "$Bin/lib";    
use Test::More;

use JSON;
use Slurp;
use Test::Rest;
use TestUtils;
use Data::Dumper;

BEGIN { use Catalyst::Test 'Bugzilla::API' }

my ($res, $req, $old, $new, $json);
my $t = Test::Rest->new('content_type' => 'application/json');
##############################################################################

plan(tests => 7 + ($put_and_get_ct * 2));

my $url = '/bug/30?' . $LOGIN;

my $oneway = slurp("$Bin/json/groups-oneway.json");
my $otherway = slurp("$Bin/json/groups-otherway.json");

##############################################################################
# Get groups
##############################################################################
$res = request($url);  
$new = from_json($res->decoded_content, {utf8 => 1});
my $orig = from_json($otherway, {utf8 => 1});
is($new->{'is_cc_accessible'}, JSON::true, "CC can see");
is($new->{'is_creator_accessible'}, JSON::true, "Reporter can see");
is_deeply($new->{'groups'}, $orig->{'groups'}, "Groups OK at the start");

##############################################################################
# Set groups and visibility bits and check
##############################################################################
my $newjson = put_and_get($url, $oneway, "Set groups and visibility");
$new = from_json($newjson, {utf8 => 1});
$old = from_json($oneway, {utf8 => 1});

is($new->{'is_cc_accessible'}, JSON::false, "CC can't see");
is($new->{'is_creator_accessible'}, JSON::false, "Reporter can't see");
is_deeply($new->{'groups'}, $old->{'groups'}, "Groups OK");

##############################################################################
# Reset groups
##############################################################################
$newjson = put_and_get($url, $otherway, "Reset groups");
$new = from_json($newjson, {utf8 => 1});
$old = from_json($otherway, {utf8 => 1});

is_deeply($new->{'groups'}, $old->{'groups'}, "Groups OK at the end");

done_testing();
