use strict;
use warnings;

use FindBin '$Bin';
use lib "$Bin/lib";    
use Test::More;

use JSON;
use Slurp;
use Test::Rest;
use TestUtils;
use Data::Dumper;

BEGIN { use Catalyst::Test 'Bugzilla::API' }

my ($res, $req, $old, $new, $json);
my $t = Test::Rest->new('content_type' => 'application/json');
##############################################################################

plan (tests => (12 * 2));

foreach my $aid (1, 2) {
  ############################################################################
  # Get attachment metadata only
  ############################################################################
  $res = request("/attachment/$aid");

  ok($res->is_success, "Request for attachment $aid should succeed");
  $new = from_json($res->decoded_content, {utf8 => 1});
  $old = load("attachment$aid.json");

  is_deeply(canonicalize($new), canonicalize($old), 
                                       "Metadata matches for attachment $aid");

  ############################################################################
  # Get attachment with data
  ############################################################################
  foreach my $url ("/attachment/$aid?include_fields=_default,data,encoding",
                   "/attachment/$aid?attachmentdata=1") 
  {
    $res = request($url);

    ok($res->is_success, "Request for attachment $aid with data OK");
    $new = from_json($res->decoded_content, {utf8 => 1});

    is($new->{'encoding'}, "base64");
    my @datasigs = (qr!^/9j/4AAQSkZJRgA!, qr/^R0lGODlhoADtAP/);
    
    like($new->{'data'}, $datasigs[$aid - 1], 
                                            "Data correct for attachment $aid");

    delete($new->{'data'});
    delete($new->{'encoding'});

    is_deeply(canonicalize($new), canonicalize($old), 
                                   "Metadata matches for attachment+data $aid");
  }
  
  ############################################################################
  # Get attachment metadata only (logged in)
  ############################################################################
  $res = request("/attachment/$aid?" . $LOGIN);

  ok($res->is_success, "Request for attachment $aid (LI) should succeed");
  $new = from_json($res->decoded_content, {utf8 => 1});
  $old = load("attachment$aid-li.json");

  delete($old->{'token'});
  delete($new->{'token'});

  is_deeply(canonicalize($new), canonicalize($old), 
                                   "Metadata (LI) matches for attachment $aid");
}

done_testing();
