use strict;
use warnings;

use FindBin '$Bin';
use lib "$Bin/lib";    
use Test::More;

use JSON;
use Slurp;
use Test::Rest;
use TestUtils;
use Data::Dumper;

BEGIN { use Catalyst::Test 'Bugzilla::API' }

my ($res, $req, $old, $new, $json);
my $t = Test::Rest->new('content_type' => 'application/json');
##############################################################################

plan(tests => 12);

##############################################################################
# Get attachments metadata only
##############################################################################
$res = request('/bug/1/attachment');

ok($res->is_success, 'Request should succeed');
$new = from_json($res->decoded_content, {utf8 => 1});
$old = load("bug1.json");
$old = { 'attachments' => $old->{'attachments'} };

is_deeply(canonicalize($new), canonicalize($old), "Metadata matches");

##############################################################################
# Get attachments with data
##############################################################################
foreach my $url ('/bug/1/attachment?attachmentdata=1',
                 '/bug/1/attachment?include_fields=_default,data,encoding') 
{
  $res = request($url);

  ok($res->is_success, 'Request should succeed');
  $new = from_json($res->decoded_content, {utf8 => 1});

  my $attach0 = $new->{'attachments'}->[1];
  is($attach0->{'encoding'}, "base64");
  like($attach0->{'data'}, qr/^R0lGODlhoADtAP/, "Attachment 2 data correct");

  delete($new->{'attachments'}->[0]->{'data'});
  delete($new->{'attachments'}->[1]->{'data'});
  delete($new->{'attachments'}->[0]->{'encoding'});
  delete($new->{'attachments'}->[1]->{'encoding'});

  is_deeply(canonicalize($new), canonicalize($old), "Metadata w/data matches");
}

##############################################################################
# Get attachments metadata only (logged in)
##############################################################################
$res = request('/bug/1/attachment?' . $LOGIN);

ok($res->is_success, 'Request should succeed');
$new = from_json($res->decoded_content, {utf8 => 1});
$old = load("bug1-li.json");
$old = { 'attachments' => $old->{'attachments'} };

delete($old->{'attachments'}->[0]->{'update_token'});
delete($old->{'attachments'}->[1]->{'update_token'});
delete($new->{'attachments'}->[0]->{'update_token'});
delete($new->{'attachments'}->[1]->{'update_token'});

is_deeply(canonicalize($new), canonicalize($old), "Metadata (LI) matches");

done_testing();
