# XXX Errors in 4.2: https://bugzilla.mozilla.org/show_bug.cgi?id=607361

use strict;
use warnings;

use FindBin '$Bin';
use lib "$Bin/lib";    
use Test::More;

use JSON;
use Slurp;
use Test::Rest;
use TestUtils;
use Data::Dumper;

BEGIN { use Catalyst::Test 'Bugzilla::API' }

my ($res, $req, $old, $new, $json);
my $t = Test::Rest->new('content_type' => 'application/json');
##############################################################################

my $bugid = 26;

my @dontcompare = ('id',  'update_token', 'ref', 'creation_time', 'bug_id', 'bug_ref', 'last_change_time');

my @attachrefs = ("A", "B", "C", "D");

plan(tests => (scalar(@attachrefs) * (3 + @dontcompare)) + 2);

##############################################################################
# Creation test
##############################################################################
foreach my $N (@attachrefs) {
  create_attachment($N);
}

sub create_attachment {
  my ($N) = @_;
  my $json = slurp("$Bin/json/attachment-create-$N.json");
  $old = from_json($json, {utf8 => 1});
  $req = $t->post(url => "/bug/$bugid/attachment?" . $LOGIN, 
                  data => $json);

  $res = request($req);
  is($res->code, 201, "Created code returned for $N");

  if ($res->code ne "201") {
    diag($res->decoded_content);
  }
  
  # Find attachment number
  my $created = from_json($res->decoded_content, {utf8 => 1});

  $created->{'ref'} =~ /(\d+)$/;
  $new = {};

  if ($1) {
    $res = request("/attachment/$1?attachmentdata=1&" . $LOGIN);

    ok($res->is_success, "Request should succeed for $N");
    $new = from_json($res->decoded_content, {utf8 => 1});
  }

  # Flag IDs won't match as they are unique
  foreach my $bug ($new, $old) {
    if ($bug->{'flags'}) {
      foreach my $flag (@{$bug->{'flags'}}) {
        delete($flag->{'id'});
      }
    }
  }
  
  # If we created with a comment, it's not returned (of course)
  delete($old->{'comments'});

  foreach my $field (@dontcompare) {
    ok($new->{$field}, "$field present and non-empty for $N");
    delete($old->{$field});
    delete($new->{$field});
  }

  is_deeply($new, $old, "Created attachment $N is correct");
}

# Check comment added correctly with Attachment D
$res = request("/bug/$bugid/comment");

ok($res->is_success, 'Request should succeed');
$new = from_json($res->decoded_content, {utf8 => 1});
my $text = $new->{'comments'}->[-1]->{'text'};

like($text, qr/creation of attachment D/, "Attachment D comment added OK");

done_testing();
