use strict;
use warnings;

use FindBin '$Bin';
use lib "$Bin/lib";    
use Test::More;

use JSON;
use Slurp;
use Test::Rest;
use TestUtils;
use Data::Dumper;

BEGIN { use Catalyst::Test 'Bugzilla::API' }

my ($res, $req, $old, $new, $json);
my $t = Test::Rest->new('content_type' => 'application/json');
##############################################################################

my $version = get_bugzilla_version();
my $basefilename = ($version >= 3.4) ? "bug1comments" : "bug1comments-legacy";

plan(tests => 4 + (($version >= 3.4) ? 3 : 0));

##############################################################################
# Comments
##############################################################################
$res = request('/bug/1/comment');

ok($res->is_success, 'Request should succeed');
$json = from_json($res->decoded_content, {utf8 => 1});

my $bug = load("$basefilename.json");

is_deeply(canonicalize($json), canonicalize($bug), 
          "Bug 1 comments are correct");

##############################################################################
# Comments, logged in (includes 1 private comment)
##############################################################################
$res = request('/bug/1/comment?' . $LOGIN);
$json = from_json($res->decoded_content, {utf8 => 1});

$bug = load("$basefilename-li.json");

is_deeply(canonicalize($json), canonicalize($bug), 
          "Bug 1 comments (logged in) are correct");

##############################################################################
# Bug with no visible comments
##############################################################################
$res = request('/bug/16/comment');
$json = from_json($res->decoded_content, {utf8 => 1});

is_deeply($json, { 'comments' => [] }, "Bug 16 no comments OK");

##############################################################################
# new_since parameter
##############################################################################
if ($version >= 3.4) {
  $res = request('/bug/1/comment?new_since=2009-09-23T00:00:00Z&' . $LOGIN);

  ok($res->is_success, 'Request should succeed');
  $json = from_json($res->decoded_content, {utf8 => 1});
  canonicalize($json);

  $bug = load("$basefilename-li.json");
  canonicalize($bug);
  
  if ($STRIP_TIMES) {
    # Must remove timestamps due to timezone issues
    foreach my $comments ($json->{'comments'}, $bug->{'comments'}) {
      foreach my $comment (@$comments) {
        delete($comment->{'creation_time'});
        delete($comment->{'last_change_time'});
      }
    }
  }
  
  # Split out comments we expect
  my @slice = splice(@{$bug->{'comments'}}, 1, 3);
  is_deeply($json->{'comments'}, \@slice, "Bug 1 comments (date) OK");

  $res = request('/bug/1/comment?new_since=2009-09-23T11:01:00Z&' . $LOGIN);
  $json = from_json($res->decoded_content, {utf8 => 1});
  canonicalize($json);

  if ($STRIP_TIMES) {
    # Must remove timestamps due to timezone issues
    foreach my $comment (@{ $json->{'comments'} }) {
      delete($comment->{'creation_time'});
      delete($comment->{'last_change_time'});
    }
  }

  @slice = splice(@slice, 1, 2);
  is_deeply($json->{'comments'}, \@slice, "Bug 1 comments (date/time) OK");
}

done_testing();
