use strict;
use warnings;

use FindBin '$Bin';
use lib "$Bin/lib";    
use Test::More;

use JSON;
use Slurp;
use Test::Rest;
use TestUtils;
use Data::Dumper;

BEGIN { use Catalyst::Test 'Bugzilla::API' }

my ($res, $req, $old, $new, $json);
my $t = Test::Rest->new('content_type' => 'application/json');
##############################################################################

my $add_comment_ct = 3;

plan(tests => 2 + ($add_comment_ct * 4));

##############################################################################
# Create comments on new bug
##############################################################################
# File new bug
$json = slurp("$Bin/json/create-A.json");
$req = $t->post(url => '/bug?' . $LOGIN, 
                data => $json);

$res = request($req);
is($res->code, 201, "Created code returned");

# Find bug number
my $created = from_json($res->decoded_content, {utf8 => 1});
$created->{'ref'} =~ /(\d+)$/;
like($1, qr/^\d+$/, "Found bug number");
my $bugid = $1;

foreach my $n (1, 2, 3) {
  $json = slurp("$Bin/json/comment-create-$n.json");
  add_comment($json, $LOGIN);
}

$json = slurp("$Bin/json/comment-create-1.json");
add_comment($json, 'username=wilma@gerv.net&password=wilmawilma');

sub add_comment {
  my ($json, $login) = @_;
  $old = from_json($json, {utf8 => 1});

  $req = $t->post(url => "/bug/$bugid/comment?" . $login, 
                  data => $json);

  $res = request($req);
  is($res->code, 201, "Created code returned");
  
  $res = request("/bug/$bugid/comment?" . $login);

  ok($res->is_success, 'Request should succeed');
  $new = from_json($res->decoded_content, {utf8 => 1});  

  is($new->{'comments'}->[-1]->{'text'}, $old->{'text'}, "Comment text OK");
}

done_testing();
