use strict;
use warnings;

use FindBin '$Bin';
use lib "$Bin/lib";    
use Test::More;

use JSON;
use Slurp;
use Test::Rest;
use TestUtils;
use Data::Dumper;

BEGIN { use Catalyst::Test 'Bugzilla::API' }

my ($res, $req, $old, $new, $json);
my $t = Test::Rest->new('content_type' => 'application/json');
##############################################################################

plan(tests => 6);

##############################################################################
# List of users
##############################################################################
$res = request('/user?match=Rubble&' . $LOGIN);

ok($res->is_success, 'Request should succeed');
$json = from_json($res->decoded_content, {utf8 => 1});

my $user = load("user-rubble.json");

is_deeply($json, $user, "Both Rubbles returned by search");

##############################################################################
# No matches
##############################################################################
$res = request('/user?match=FlorbleZorp&' . $LOGIN);

ok($res->is_success, 'Request should succeed');
$json = from_json($res->decoded_content, {utf8 => 1});

is_deeply($json, { "users" => [] }, "No matches");

##############################################################################
# Not logged in
##############################################################################
$res = request('/user?match=Rubble');

$json = from_json($res->decoded_content, {utf8 => 1});

is($json->{'error'}, JSON::true, "Not logged in leads to error");
is($json->{'code'}, 505, "Not logged in leads to correct error");

done_testing();
