use strict;
use warnings;

use FindBin '$Bin';
use lib "$Bin/lib";    
use Test::More;

use JSON;
use Slurp;
use Test::Rest;
use TestUtils;
use Data::Dumper;

BEGIN { use Catalyst::Test 'Bugzilla::API' }

my ($res, $req, $old, $new, $json);
my $t = Test::Rest->new('content_type' => 'application/json');
##############################################################################

my $url = '/configuration';

my @hashes = ("classification", "product", "field", "flag_type", "group");
my @scalars = ("maintainer", "version", "announcement", "max_attachment_size");

plan(tests => 3 * (@hashes + @scalars + 2));

##############################################################################
# Get config - twice (may one day invoke 304 Not Modified)
##############################################################################
for (my $i = 0; $i < 2; $i++) {
  $res = request($url);
  ok($res->is_success, 'Request should succeed');

  $new = from_json($res->decoded_content, {utf8 => 1});
  is($new->{'maintainer'}, 'gerv@mozilla.org', "Maintainer OK");

  foreach my $thing (@hashes) { 
    is(ref($new->{$thing}), "HASH", "$thing is an HASH");
  }

  foreach my $thing (@scalars) {
    ok(defined(($new->{$thing})), "$thing is present");
  }
}

##############################################################################
# Get config with no flags
##############################################################################
$url = '/configuration?flags=0';
$res = request($url);
ok($res->is_success, 'Request should succeed');

$new = from_json($res->decoded_content, {utf8 => 1});
is($new->{'maintainer'}, 'gerv@mozilla.org', "Maintainer OK");

foreach my $thing (grep { $_ ne "flag_type" } @hashes) { 
  is(ref($new->{$thing}), "HASH", "$thing is an HASH");
}

is($new->{'flag_type'}, undef, "flag_type not present");

foreach my $thing (@scalars) {
  ok(defined(($new->{$thing})), "$thing is present");
}

done_testing();
