use strict;
use warnings;

use FindBin '$Bin';
use lib "$Bin/lib";    
use Test::More;

use JSON;
use Slurp;
use Test::Rest;
use TestUtils;
use Data::Dumper;

BEGIN { use Catalyst::Test 'Bugzilla::API' }

my ($res, $req, $old, $new, $json);
my $t = Test::Rest->new('content_type' => 'application/json');
##############################################################################

my $url = '/count';
my $SAMLOGIN = '&username=sam@gerv.net&password=samsam';

my %nobugsqueries = (
  "?product=NonExistent" => 0,
  "?product=NonExistent&x_axis_field=priority" => [],
  "?product=NonExistent&x_axis_field=priority&y_axis_field=severity" => [[]],
  "?product=NonExistent&x_axis_field=priority&y_axis_field=severity" . 
    "&z_axis_field=component" => [[[]]]
);

plan(tests => 15 + (keys(%nobugsqueries) * 2));

##############################################################################
# 0-dimensional
##############################################################################
$res = request($url);
ok($res->is_success, 'Request should succeed');
$new = from_json($res->decoded_content, {utf8 => 1});
cmp_ok($new->{'data'}, '>', 300, "Lots of bugs in the database");
cmp_ok($new->{'data'}, '<', 20000, "Not that many bugs in the database");

##############################################################################
# 1-dimensional
##############################################################################
$old = load("count-1d.json");
$res = request($url . "?product=Bedrock&x_axis_field=severity&severity=blocker"
                    . "&severity=critical&severity=major");
ok($res->is_success, 'Request should succeed');
$new = from_json($res->decoded_content, {utf8 => 1});
is_deeply($new, $old, "1d data OK");

$res = request($url . "?product=Bedrock&x_axis_field=severity&severity=blocker"
                    . "&severity=critical&severity=major" . $SAMLOGIN);
ok($res->is_success, 'Request should succeed');
$new = from_json($res->decoded_content, {utf8 => 1});
# Account for bug 301, which is in the 'secret' group
$old->{'data'}->[2] += 1;
is_deeply($new, $old, "1d data OK (logged in)");

##############################################################################
# 2-dimensional
##############################################################################
$old = load("count-2d.json");
$res = request($url . "?product=Bedrock&x_axis_field=severity&severity=blocker"
                    . "&severity=critical&severity=major" 
                    . "&y_axis_field=priority");
ok($res->is_success, 'Request should succeed');
$new = from_json($res->decoded_content, {utf8 => 1});
is_deeply($new, $old, "2d data OK");

$res = request($url . "?product=Bedrock&x_axis_field=severity&severity=blocker"
                    . "&severity=critical&severity=major" 
                    . "&y_axis_field=priority" . $SAMLOGIN);
ok($res->is_success, 'Request should succeed');
$new = from_json($res->decoded_content, {utf8 => 1});
# Account for bug 301, which is in the 'secret' group
$old->{'data'}->[3]->[2] += 1;
is_deeply($new, $old, "2d data OK (logged in)");

##############################################################################
# 3-dimensional
##############################################################################
$old = load("count-3d.json");
$res = request($url . "?product=Bedrock&product=Rockville"
                    . "&x_axis_field=severity&severity=blocker"
                    . "&severity=critical&severity=major&severity=normal" 
                    . "&y_axis_field=priority&z_axis_field=component");
ok($res->is_success, 'Request should succeed');
$new = from_json($res->decoded_content, {utf8 => 1});
is_deeply($new, $old, "3d data OK");

$res = request($url . "?product=Bedrock&product=Rockville"
                    . "&x_axis_field=severity&severity=blocker"
                    . "&severity=critical&severity=major&severity=normal" 
                    . "&y_axis_field=priority&z_axis_field=component"
                    . $SAMLOGIN);
ok($res->is_success, 'Request should succeed');
$new = from_json($res->decoded_content, {utf8 => 1});
# Account for bug 301, which is in the 'secret' group
$old->{'data'}->[1]->[3]->[2] += 1;
is_deeply($new, $old, "3d data OK (logged in)");

##############################################################################
# Nothing returned
##############################################################################
foreach my $query (keys %nobugsqueries) {
  $res = request($url . $query);
  ok($res->is_success, 'Request should succeed');
  $new = from_json($res->decoded_content, {utf8 => 1});
  is_deeply($new->{'data'}, $nobugsqueries{$query}, "No bugs OK for $query");
}

done_testing();

