use strict;
use warnings;

use FindBin '$Bin';
use lib "$Bin/lib";    
use Test::More;

use JSON;
use Slurp;
use Test::Rest;
use TestUtils;
use Data::Dumper;

BEGIN { use Catalyst::Test 'Bugzilla::API' }

my ($res, $req, $old, $new, $json);
my $t = Test::Rest->new('content_type' => 'application/json');
##############################################################################

# Sample things in default set, of different types: 
#   keywords, status, assigned_to
# Sample things not in default set, of different types:
#   is_confirmed, blocks, attachments

my %buglist_urls = (
  "include_fields=_all" => "bug1-all.json",
  "include_fields=_all,id,status" => "bug1-all.json",
  "include_fields=_all,_default" => "bug1-all.json",
  "include_fields=_all,_default,id,status" => "bug1-all.json",
  "include_fields=_default" => "bug-bug1-li.json",
  "include_fields=_default,keywords,assigned_to" => "bug-bug1-li.json",
  "include_fields=_default&exclude_fields=is_confirmed,blocks" => "bug-bug1-li.json",
);

my %buglist_check_included = (
  "include_fields=_default,is_confirmed,blocks" => ["keywords", "status", "is_confirmed", "blocks"],
  "include_fields=id,status,assigned_to" => ["id", "status", "assigned_to"],
  "include_fields=id,is_confirmed,blocks" => ["id", "is_confirmed", "blocks"],
  "include_fields=_all" => ["history"],
);

my %buglist_check_excluded = (
  "include_fields=id,status,assigned_to" => ["blocks", "keywords"],
  "include_fields=is_confirmed,blocks" => ["id"],
  "include_fields=_default&exclude_fields=keywords,status" => ["keywords", "status"],
  "exclude_fields=keywords,status" => ["is_confirmed", "blocks"],
  "exclude_fields=cf_single,blocks" => ["cf_single", "blocks"],
);

# Manual count :-|
plan(tests => 58);

##############################################################################
# Buglists
##############################################################################
sub get_and_check {
  my ($baseurl, $queries, $mode) = @_;
  
  foreach my $query (keys %$queries) {
    $res = request($baseurl . $query . "&" . $LOGIN);

    ok($res->is_success, 'Request should succeed');
    $new = from_json($res->decoded_content, {utf8 => 1});

    my $bug = $new->{'bugs'}->[0];

    # Allow for lists to have more than one bug - but we need ID in that case
    if (scalar (@{ $new->{'bugs'} }) > 1) {
        my @bug1s = grep { $_->{'id'} == 1 } @{ $new->{'bugs'} };
        $bug = $bug1s[0];
    }
    
    $bug = canonicalize($bug);
    
    if ($mode eq "FILE") {
      my $file = $queries->{$query};
      $old = load("$file");
      
      # Some files are a bugs array, some are a straight bug
      $old = $old->{'bugs'} ? $old->{'bugs'}->[0] : $old;
      
      $old = canonicalize($old);
    
      is_deeply($bug, $old, "File matches OK for $query");
    }
    elsif ($mode eq "EXPECTED") {
      foreach my $field (@{ $queries->{$query} }) {
        isnt($bug->{$field}, undef, "$field present for $query");
      }
    }
    elsif ($mode eq "UNEXPECTED") {
      foreach my $field (@{ $queries->{$query} }) {
        is($bug->{$field}, undef, "$field not present for $query");
      }
    }
    else {
      is(1, 0, "Invalid mode specified");
    }
  }
}

get_and_check("/bug?id=1&", \%buglist_urls, "FILE");
get_and_check("/bug?id=1&", \%buglist_check_included, "EXPECTED");
get_and_check("/bug?id=1,2,3,4,5,6&", \%buglist_check_included, "EXPECTED");
get_and_check("/bug?id=1&", \%buglist_check_excluded, "UNEXPECTED");

done_testing();
