#
# Rest.pm
# Created by: Adam Jacob, Marchex, <adam@hjksolutions.com>
# Created on: 10/16/2006 11:11:25 AM PDT
#
# $Id: $

# This file is from the Catalyst::Controller::REST distribution
# http://cpansearch.perl.org/src/BOBTFISH/Catalyst-Action-REST-0.76/t/lib/Test/Rest.pm
# License: "You may distribute this code under the same terms as Perl itself."

package Test::Rest;

use strict;
use warnings;

use LWP::UserAgent;
use Params::Validate qw(:all);

sub new {
    my $self = shift;
    my %p    = validate( @_, { 
            content_type => { type => SCALAR }, 
            accept       => { type => SCALAR, optional => 1 }, 
        }, 
    );
    my $ref  = {
        'ua'           => LWP::UserAgent->new,
        'content_type' => $p{'content_type'},
        'accept'       => $p{'accept'},
    };
    bless $ref, $self;
}

{
    my @non_data_methods = qw(GET DELETE OPTIONS);
    foreach my $method (@non_data_methods) {
        no strict 'refs';
        my $sub = lc($method);
        *$sub = sub {
            my $self = shift;
            my %p    = validate( @_, { url => { type => SCALAR }, }, );
            my $req  = HTTP::Request->new( "$method" => $p{'url'} );
            $req->content_type( $self->{'content_type'} );
            $req->header( "Accept" => $self->{'accept'} ) if $self->{'accept'};
            return $req;
        };
    }

    my @data_methods = qw(PUT POST);
    foreach my $method (@data_methods) {
        no strict 'refs';
        my $sub = lc($method);
        *{$sub} = sub {
            my $self = shift;
            my %p    = validate(
                @_,
                {
                    url  => { type => SCALAR },
                    data => 1,
                },
            );
            my $req = HTTP::Request->new( "$method" => $p{'url'} );
            $req->content_type( $self->{'content_type'} );
            $req->header( "Accept" => $self->{'accept'} ) if $self->{'accept'};
            $req->content_length(
                do { use bytes; length( $p{'data'} ) }
            );
            $req->content( $p{'data'} );
            return $req;
        };
    }
}

1;

